<?
/*
(C) 2004-2013 DatuX - info@datux.nl

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


require_once("../file.php");
require_once("../error.php");

function openvpn_GenerateConfig($name)
{
	SafeExec("cd /etc/openvpn; ./syn3-genconfig $name");
}

function openvpn_ReadSettings()
{
	return(FileReadSettings("/etc/openvpn/syn3.conf"));
}

/*
HEADER  CLIENT_LIST     Common Name     Real Address    Virtual Address Bytes Received  Bytes Sent      Connected Since Connected Since (time_t)        Username
CLIENT_LIST     test2   188.89.163.201:35963    10.8.0.18       3039    3717    Tue Nov 22 23:11:17 2016        1479852677      psy
CLIENT_LIST     test20  192.168.13.236:48928    10.8.0.34       47084   43668   Tue Nov 22 23:08:06 2016        1479852486      psy

2.4.x:
HEADER  CLIENT_LIST     Common Name     Real Address    Virtual Address Virtual IPv6 Address    Bytes Received  Bytes Sent      Connected Since Connected Since (time_t)        Username        Client ID       Peer ID
CLIENT_LIST     test    213.126.87.242:58739    10.8.0.10               17705   13762   Thu May 17 12:51:25 2018        1526561485      test    2       1


*/
function openvpn_Status()
{
	$lines=FileRead("/etc/openvpn/openvpn-status.log");
	$ret=[];
	foreach ($lines as $line)
	{
		$fields=explode("\t",$line);
		if ($fields[0]=="CLIENT_LIST")
			$ret[]=$fields;
	}
	return($ret);
}

function openvpn_WriteSettings($data)
{
	if (!ValidHostname($data['SERVER']) && !ValidIP($data['SERVER']))
		return (array("SERVER"=>_InvalidHostname()));


	FileWriteSettings("/etc/openvpn/syn3.conf", $data);

    //generate additional config settings
    $cfg=Array();
    $cfg[]="### Additional SYN-3 OpenVPN settings";
    $cfg[]="# will be overwritten when settings are saved in SCC";

    if ($data['ROUTE_MODE']=="all")
        $cfg[]="push 'redirect-gateway'";
    else
    {
        if ($data['SUBNET'])
        {
            $err=Error_ValidSubnet($data['SUBNET']);
            if ($err)
                return (array("SUBNET"=>$err));

            list($ip,$bits)=explode("/", $data['SUBNET']);
            $netmask=CalcMask($bits);
            $cfg[]="push 'route $ip $netmask'";
        }
    }

    if ($data['PUSH_DNS']!="")
    {
        $err=Error_ValidIP($data['PUSH_DNS']);
        if ($err)
            return (array("PUSH_DNS"=>$err));

        $cfg[]="push 'dhcp-option DNS ".$data['PUSH_DNS']."'";
    }


    FileWrite("/etc/openvpn/openvpn_syn3.conf",$cfg);
    LogInfo(_OpenVPNSettingsChanged());

    SvcRestart("openvpn");

}

function openvpn_Create($name,$data)
{

	if (!ValidFilename($name))
		return (array("name"=>_InvalidCertificateName()));


	if ($data['key']!="" && strlen($data['key'])<4)
		return (array("key"=>_CertKeySizeTooSmall(4)));

	if (openvpn_List()[$name])
		return (array("name"=>_OpenVPNCertAlreadyExists()));

	Progress(_CreatingCertificate());
	SafeExec("cd /etc/openvpn; ./syn3-client $name nopass");
	if ($data['key']!="")
	{
		//this is the only way to pass the password
		SafeExec("openssl rsa -passout pass:'".$data['key']."' -in /etc/openvpn/pki/private/$name.key -out /etc/openvpn/pki/private/$name.key -des3");
		openvpn_GenerateConfig($name);
	}

	//log what just happend:
	LogInfo(_CertificateCreated($name));
}


function openvpn_Revoke($name)
{
	if (!ValidFilename($name))
		Error(_InvalidCertificateName());

	SafeExec("cd /etc/openvpn; echo yes | ./easyrsa revoke $name");
	SafeExec("cd /etc/openvpn; ./easyrsa gen-crl");
	LogInfo(_CertificateRevokedd($name));

}


/** Gets a list of all the items
 * $filter: search for this textstring in all the items
 * @return $ret[itemnr]=array(itemdata)
 */
function openvpn_List($filter="")
{
	$lines=FileRead("/etc/openvpn/pki/index.txt");
	$ret=[];
	foreach ($lines as $line)
	{
		list($status, $id, $revokedate, $serial, $huh, $cn) = explode("\t",$line);
		list($bla, $name) = explode("=", $cn);
		if ($filter=="" || strpos($name,$filter)>-1 )
			$ret[$name]="$status";
	}
	// echo "<pre>";print_r($ret);
	return ($ret);
}
